@@ -6,6 +6,26 @@
 
 namespace apg {
 
+    int reportCudaError(cudaError_t err) {
+
+        if (err != cudaSuccess) {
+            std::cerr << "\033[31;1mCUDA Error " << ((int) err) << " : " <<
+                cudaGetErrorString(err) << "\033[0m" << std::endl;
+
+            return ((int) err);
+        }
+
+        return 0;
+    }
+
+    int getDeviceCount() {
+
+        int count = 0;
+        if (reportCudaError(cudaGetDeviceCount(&count))) { return -1; }
+        return count;
+
+    }
+
     void GpuSearcher::pump(std::string seed, uint64_t epoch, std::vector<uint64_t> &vec) {
 
         int minibatch = this->num_universes;
@@ -14,7 +34,7 @@ namespace apg {
         hash_container *hc = (hash_container*) this->xhc;
         uint4 *multiverse = (uint4*) this->xmc;
 
-        cudaSetDevice(this->device);
+        reportCudaError(cudaSetDevice(this->device));
 
         hc->create_hashes(seed, epoch);
 
@@ -36,16 +56,18 @@ namespace apg {
         // for (auto&& it : vec) { std::cout << it << ", "; }
         // std::cout << std::endl;
 
-        cudaError_t err = cudaGetLastError();
-        if (err != cudaSuccess) { std::cerr << "Error: " << cudaGetErrorString(err) << std::endl; }
+        reportCudaError(cudaGetLastError());
     }
 
-    GpuSearcher::GpuSearcher(int dev, int num_universes, std::string symmetry) {
+    GpuSearcher::GpuSearcher(int dev, int unused_unicount, std::string symmetry) {
 
+        reportCudaError(cudaSetDevice(dev));
+
+        (void) unused_unicount;
         size_t free_mem = 0;
         size_t total_mem = 0;
 
-        cudaMemGetInfo(&free_mem, &total_mem);
+        reportCudaError(cudaMemGetInfo(&free_mem, &total_mem));
 
         std::cerr << "Memory statistics: " << free_mem << " free; " << total_mem << " total." << std::endl;
 
@@ -74,15 +96,13 @@ namespace apg {
 
         this->xhc = (void*) hc;
 
-        cudaSetDevice(dev);
-
-        hc->spin_up(true, num_universes);
+        hc->spin_up(true);
     }
 
     GpuSearcher::~GpuSearcher() {
         hash_container *hc = (hash_container*) this->xhc;
 
-        cudaSetDevice(this->device);
+        reportCudaError(cudaSetDevice(this->device));
 
         hc->tear_down();
         cudaFree(this->xmc);
