@@ -338,27 +338,21 @@ extern "C"
     void GetWechslerOfPattern(void* pat, char* buffer, int buflen) {
         auto ppat = reinterpret_cast<apg::pattern*>(pat);
         std::string s = ppat->phase_wechsler(999999999);
+        apg::try_copy(s, buffer, buflen);
+    }
 
-        if (s.length() >= buflen) {
-            std::ostringstream ss;
-            ss << "!" << (((s.length() >> 8) + 1) << 8);
-            s = ss.str();
-        }
-
-        s.copy(buffer, buflen);
+    void GetRleOfPattern(void* pat, char* buffer, int buflen) {
+        auto ppat = reinterpret_cast<apg::pattern*>(pat);
+        std::ostringstream ss;
+        ppat->write_rle(ss);
+        std::string s = ss.str();
+        apg::try_copy(s, buffer, buflen);
     }
 
     void GetApgcodeOfPattern(void* pat, char* buffer, int buflen) {
         auto ppat = reinterpret_cast<apg::pattern*>(pat);
         std::string s = ppat->apgcode(999999999, 999999999);
-
-        if (s.length() >= buflen) {
-            std::ostringstream ss;
-            ss << "!" << (((s.length() >> 8) + 1) << 8);
-            s = ss.str();
-        }
-
-        s.copy(buffer, buflen);
+        apg::try_copy(s, buffer, buflen);
     }
 
     void GetCompiledVersion(char* buffer) {
