@@ -5,6 +5,7 @@ from .crt import chinese_remainder, large_primes
 from .samples import download_synthesis as dsynth
 from .samples import catagolue_url
 from .registry import register_pattern_callback
+from .idgun import determine_if_gun
 
 def randfill(item, shape):
     '''
@@ -489,7 +490,7 @@ class Pattern(object):
             x = self.lifelib(fn, self.ptr, [buflen], buflen)
         return x
 
-    def oscar(self, maxexp=24, eventual_oscillator=True, verbose=True, return_apgcode=False):
+    def oscar(self, maxexp=16, eventual_oscillator=True, verbose=True, return_apgcode=False, allow_guns=True):
 
         logdiam = self.lifelib('GetDiameterOfPattern', self.ptr)
 
@@ -508,6 +509,12 @@ class Pattern(object):
             if (eventual_oscillator):
                 x = y
 
+        if return_apgcode and allow_guns and self.getrule() == 'b3s23':
+            rv = determine_if_gun(self, maxexp)
+            if rv is not None:
+                cg, multiple, cw, z, envelope = rv
+                return {'period': multiple, 'displacement': tuple([-x for x in z.displacement]), 'apgcode': 'xg%d_%s' % (multiple, cw)}
+
         if (verbose):
             print("Oscillation not detected (try increasing maxexp?)")
         return {}
@@ -921,11 +928,7 @@ observer.observe(targetNode, config);
 
         return float(centre[0]), float(centre[1]), float(pixelsize)
 
-    def make_gif(self, hue='#ff0000', width=350, height=350, border=5):
-        '''
-        Intelligently create an animated GIF of an oscillator or spaceship.
-        Spaceships will be oriented to point between north and north-west.
-        '''
+    def make_gif_internal(self, p, disp, hue='#ff0000', width=350, height=350, border=5, bg=None):
 
         import numpy as np
         saturation = 1.0
@@ -939,10 +942,30 @@ observer.observe(targetNode, config);
             lh = lh // 3
             hue = np.array([int(hue[lh*i:lh*(i+1)], 16) for i in range(3)])
             hue = hue / (16 ** lh - 1.0)
-            saturation = np.max(np.abs(hue * 2 - 1))
+            saturation = np.max(hue) - np.min(hue)
+
+        if bg is not None:
+            params = bg.centre_and_pixel_size(1, (0, 0), width - 2 * border, height - 2 * border)
+        else:
+            params = self.centre_and_pixel_size(p, disp, width - 2 * border, height - 2 * border)
+
+        frames_per_gen = max(2, min(240 // p, 8))
+        frames = frames_per_gen * p
+
+        anim = self.animate(frames, p, disp, params, width, height)
+        anim2 = [colourise_image(x, hue, saturation) for x in blur_animation(anim, 2)]
+
+        return anim2
+        
+
+    def make_gif(self, filename=None, fps=25, **kwargs):
+        '''
+        Intelligently create an animated GIF of an oscillator or spaceship.
+        Spaceships will be oriented to point between north and north-west.
+        '''
 
         y = self
-        d = y.oscar(maxexp=16, verbose=False)
+        d = y.oscar(maxexp=16, return_apgcode=True, verbose=False)
 
         disp = list(d['displacement'])
         p = d['period']
@@ -956,14 +979,27 @@ observer.observe(targetNode, config);
             disp[0] = -disp[0]
 
         if disp[1] > 0:
-            y = y.transform('flip_x')
+            y = y.transform('flip_y')
             disp[1] = -disp[1]
 
-        params = y.centre_and_pixel_size(p, disp, width - 2 * border, height - 2 * border)
-        frames_per_gen = max(2, min(240 // p, 8))
-        frames = frames_per_gen * p
+        bg = None
 
-        anim = y.animate(frames, p, disp, params, width, height)
-        anim2 = [colourise_image(x, hue, saturation) for x in blur_animation(anim, 2)]
+        if d['apgcode'].startswith('xg'):
+            disp = (0, 0)
+            bg = determine_if_gun(y, 16)[4]
+            rect = bg.getrect()
+            periods = 1 + (4 * (rect[2] + rect[3])) // p
+            y = y[p * periods]
 
-        return anim2
+            if (rect[3] > rect[2]):
+                y = y.transform('swap_xy')
+                bg = bg.transform('swap_xy')
+
+        gif = y.make_gif_internal(p, disp, bg=bg, **kwargs)
+
+        if filename is None:
+            return gif
+
+        import imageio
+        imageio.mimsave(filename, gif, fps=fps)
+        return d['apgcode']
