@@ -3,6 +3,8 @@ import re
 
 from collections import Sequence, defaultdict
 
+catagolue_url = "https://catagolue.hatsya.com"
+
 def get_symmetries(rulestring):
 
     validsyms = ["1x256", "2x128", "4x64", "8x32", "C1"] # asymmetric soups
@@ -70,7 +72,7 @@ class SampleSoupList(Sequence):
     def __repr__(self):
         return 'SampleSoupList(%r, %r, %r, %r)' % (self.lt, self.rule, self.symmetry, self.samples)
 
-def download_synthesis(apgcode, rule, domain='https://gol.hatsya.co.uk', tempfile='tempfile'):
+def download_synthesis(apgcode, rule, domain=catagolue_url, tempfile='tempfile'):
 
     if ('_' in rule) and ('_' not in apgcode):
         rule, apgcode = apgcode, rule
@@ -91,7 +93,7 @@ def download_synthesis(apgcode, rule, domain='https://gol.hatsya.co.uk', tempfil
     else:
         return synthesis
 
-def download_samples(lt, apgcode, rule, domain='https://gol.hatsya.co.uk', tempfile='tempfile'):
+def download_samples(lt, apgcode, rule, domain=catagolue_url, tempfile='tempfile'):
 
     if ('_' in rule) and ('_' not in apgcode):
         rule, apgcode = apgcode, rule
